package sample;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

public class SwingBulkTable extends JTable {

	private static final long serialVersionUID = 1L;
	
	private final DefaultTableModel tableModel;
	
	public static class SymbolCell implements Comparable<SymbolCell> {
		public Icon icon;
		public String symbol;
		
		public SymbolCell(Icon icon, String symbol) {
			this.icon = icon;
			this.symbol  = symbol;
		}

		@Override
		public int compareTo(SymbolCell o) {
			return this.symbol.compareTo(o.symbol);
		}
	}
	
	private static class SymbolCellRenderer extends DefaultTableCellRenderer {
		private static final long serialVersionUID = 1L;

		@Override
		public void setValue(Object value) {
			if (value == null) {
				return;
			}
			SymbolCell symCell = (SymbolCell)value;
			setIcon(symCell.icon);
			setText(symCell.symbol);
		}
	}

	private static class SwingBulkTableModel extends DefaultTableModel {
		private static final long serialVersionUID = 1L;

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			if (columnIndex == 0)
				return SymbolCell.class;
			else
				return String.class;
		}
	}
	
	public SwingBulkTable() {
		tableModel = new SwingBulkTableModel();
		tableModel.setColumnIdentifiers(new String[] {"Symbol", "Type", "Ask", "Bid"});
		
		super.setModel(tableModel);
		super.setDefaultRenderer(SymbolCell.class, new SymbolCellRenderer());
		super.setAutoCreateRowSorter(true);
		super.setEnabled(false);
		
		super.getColumnModel().getColumn(0).setPreferredWidth(150);
		super.getColumnModel().getColumn(1).setPreferredWidth(100);
		super.getColumnModel().getColumn(2).setPreferredWidth(150);
		super.getColumnModel().getColumn(3).setPreferredWidth(150);
	}
	
	public SymbolCell getSymbolCellAt(int rowIndex, int columnIndex) {
		return (SymbolCell)tableModel.getValueAt(rowIndex, columnIndex);
	}
	
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		tableModel.setValueAt(value, rowIndex, columnIndex);
	}
	
	public void setRow(int rowIndex, String symbol, Icon icon, String msgType,
			String ask, String bid) {
		SymbolCell symCell = new SymbolCell(icon, symbol);
		
		tableModel.setValueAt(symCell, rowIndex, 0);
		tableModel.setValueAt(msgType, rowIndex, 1);
		tableModel.setValueAt(ask, rowIndex, 2);
		tableModel.setValueAt(bid, rowIndex, 3);
	}
	
	public void setRow(int rowIndex, String symbol, Icon icon, String msgType) {
		SymbolCell symCell = new SymbolCell(icon, symbol);
		
		tableModel.setValueAt(symCell, rowIndex, 0);
		tableModel.setValueAt(msgType, rowIndex, 1);
	}
	
	public void setCell(int rowIndex, int colIndex, String val) {
		tableModel.setValueAt(val, rowIndex, colIndex);
	}
	
	@Override
	public int getRowCount() {
		return tableModel.getRowCount();
	}
	
	public void setRowCount(int rowCount) {
		tableModel.setRowCount(rowCount);
	}
}
